@@ -1,47 +1,44 @@
 #!/bin/sh
 
 #############################################################################
-# configuration to fill in (or to replace in your .staticperlrc)
+# configuration to fill in
 
 STATICPERL=~/.staticperl
 CPAN=http://mirror.netcologne.de/cpan # which mirror to use
 EMAIL="read the documentation <rtfm@example.org>"
-DLCACHE=
 
 # perl build variables
 MAKE=make
-PERL_VERSION=5.12.4 # 5.8.9 is also a good choice
+PERL_VERSION=5.12.3 # 5.8.9 is also a good choice
 PERL_CC=cc
 PERL_CONFIGURE="" # additional Configure arguments
 PERL_CCFLAGS="-g -DPERL_DISABLE_PMC -DPERL_ARENA_SIZE=16376 -DNO_PERL_MALLOC_ENV -D_GNU_SOURCE -DNDEBUG"
-PERL_OPTIMIZE="-Os" # -Os -ffunction-sections -fdata-sections -finline-limit=8 -ffast-math"
+PERL_OPTIMIZE="-Os -ffunction-sections -fdata-sections -finline-limit=8 -ffast-math"
 
 ARCH="$(uname -m)"
 
-#case "$ARCH" in
-#   i*86 | x86_64 | amd64 )
-#      PERL_OPTIMIZE="$PERL_OPTIMIZE -mpush-args -mno-inline-stringops-dynamically -mno-align-stringops -mno-ieee-fp" # x86/amd64
-#      case "$ARCH" in
-#         i*86 )
-#            PERL_OPTIMIZE="$PERL_OPTIMIZE -fomit-frame-pointer -march=pentium3 -mtune=i386" # x86 only
-#            ;;
-#      esac
-#      ;;
-#esac
+case "$ARCH" in
+   i*86 | x86_64 | amd64 )
+      PERL_OPTIMIZE="$PERL_OPTIMIZE -mpush-args -mno-inline-stringops-dynamically -mno-align-stringops -mno-ieee-fp" # x86/amd64
+      case "$ARCH" in
+         i*86 )
+            PERL_OPTIMIZE="$PERL_OPTIMIZE -fomit-frame-pointer -march=pentium3 -mtune=i386" # x86 only
+            ;;
+      esac
+      ;;
+esac
 
 # -Wl,--gc-sections makes it impossible to check for undefined references
 # for some reason so we need to patch away the "-no" after Configure and before make :/
 # --allow-multiple-definition exists to work around uclibc's pthread static linking bug
-#PERL_LDFLAGS="-Wl,--no-gc-sections -Wl,--allow-multiple-definition"
-PERL_LDFLAGS=
+PERL_LDFLAGS="-Wl,--no-gc-sections -Wl,--allow-multiple-definition"
 PERL_LIBS="-lm -lcrypt" # perl loves to add lotsa crap itself
 
 # some configuration options for modules
 PERL_MM_USE_DEFAULT=1
-PERL_MM_OPT="MAN1PODS= MAN3PODS="
 #CORO_INTERFACE=p # needed without nptl on x86, due to bugs in linuxthreads - very slow
-#EV_EXTRA_DEFS='-DEV_FEATURES=4+8+16+64 -DEV_USE_SELECT=0 -DEV_USE_POLL=1 -DEV_USE_EPOLL=1 -DEV_NO_LOOPS -DEV_COMPAT3=0'
-export PERL_MM_USE_DEFAULT PERL_MM_OPT
+EV_EXTRA_DEFS='-DEV_FEATURES=4+8+16+64 -DEV_USE_SELECT=0 -DEV_USE_POLL=1 -DEV_USE_EPOLL=1 -DEV_NO_LOOPS -DEV_COMPAT3=0'
+export PERL_MM_USE_DEFAULT CORO_INTERFACE EV_EXTRA_DEFS
 
 # which extra modules to install by default from CPAN that are
 # required by mkbundle
@@ -52,7 +49,6 @@ EXTRA_MODULES=""
 
 # overridable functions
 preconfigure()  { : ; }
-patchconfig()   { : ; }
 postconfigure() { : ; }
 postbuild()     { : ; }
 postinstall()   { : ; }
@@ -69,19 +65,15 @@ fi
 #############################################################################
 # support
 
+MKBUNDLE="${MKBUNDLE:=$STATICPERL/mkbundle}"
 PERL_PREFIX="${PERL_PREFIX:=$STATICPERL/perl}" # where the perl gets installed
 
-unset PERL5OPT PERL5LIB PERLLIB PERL_UNICODE PERLIO_DEBUG
-unset PERL_MB_OPT
+unset PERL5OPT PERL5LIB PERLLIB PERL_UNICODE PERLIO_DEBUG 
 LC_ALL=C; export LC_ALL # just to be on the safe side
 
-# prepend PATH - not required by staticperl itself, but might make
-# life easier when working in e.g. "staticperl cpan / look"
-PATH="$PERL_PREFIX/perl/bin:$PATH"
-
 # set version in a way that Makefile.PL can extract
 VERSION=VERSION; eval \
-$VERSION="1.4"
+$VERSION="1.11"
 
 BZ2=bz2
 BZIP2=bzip2
@@ -134,7 +126,7 @@ EOF
 # download/configure/compile/install perl
 
 clean() {
-   rm -rf "$STATICPERL/src"
+   rm -rf "$STATICPERL/src/perl-$PERL_VERSION"
 }
 
 realclean() {
@@ -150,36 +142,23 @@ fetch() {
    rcd src
 
    if ! [ -d "perl-$PERL_VERSION" ]; then
-      PERLTAR=perl-$PERL_VERSION.tar.$BZ2
+      if ! [ -e "perl-$PERL_VERSION.tar.$BZ2" ]; then
 
-      if ! [ -e $PERLTAR ]; then
-         URL="$CPAN/src/5.0/$PERLTAR"
+         URL="$CPAN/src/5.0/perl-$PERL_VERSION.tar.$BZ2"
 
          verblock <<EOF
 downloading perl
 to manually download perl yourself, place
 perl-$PERL_VERSION.tar.$BZ2 in $STATICPERL
 trying $URL
-
-either curl or wget is required for automatic download.
-curl is tried first, then wget.
-
-you can configure a download cache directory via DLCACHE
-in your .staticperlrc to avoid repeated downloads.
 EOF
 
-         rm -f $PERLTAR~ # just to be on the safe side
-         { [ "$DLCACHE" ] && cp "$DLCACHE"/$PERLTAR $PERLTAR~ >/dev/null 2>&1; } \
-            || wget -O $PERLTAR~ "$URL" \
-            || curl -f >$PERLTAR~ "$URL" \
+         rm -f perl-$PERL_VERSION.tar.$BZ2~ # just to be on the safe side
+         curl -f >perl-$PERL_VERSION.tar.$BZ2~ "$URL" \
+            || wget -O perl-$PERL_VERSION.tar.$BZ2~ "$URL" \
             || fatal "$URL: unable to download"
-         rm -f $PERLTAR
-         mv $PERLTAR~ $PERLTAR
-         if [ "$DLCACHE" ]; then
-            mkdir -p "$DLCACHE"
-            cp $PERLTAR "$DLCACHE"/$PERLTAR~ && \
-               mv "$DLCACHE"/$PERLTAR~ "$DLCACHE"/$PERLTAR
-         fi
+         rm -f perl-$PERL_VERSION.tar.$BZ2
+         mv perl-$PERL_VERSION.tar.$BZ2~ perl-$PERL_VERSION.tar.$BZ2
       fi
 
       verblock <<EOF
@@ -188,8 +167,8 @@ EOF
 
       mkdir -p unpack
       rm -rf unpack/perl-$PERL_VERSION
-      $BZIP2 -d <$PERLTAR | ( cd unpack && tar xf - ) \
-         || fatal "$PERLTAR: error during unpacking"
+      $BZIP2 -d <perl-$PERL_VERSION.tar.$BZ2 | tar xfC - unpack \
+         || fatal "perl-$PERL_VERSION.tar.$BZ2: error during unpacking"
       chmod -R u+w unpack/perl-$PERL_VERSION
       mv unpack/perl-$PERL_VERSION perl-$PERL_VERSION
       rmdir -p unpack
@@ -250,11 +229,7 @@ EOF
    grep -q -- -fstack-protector Configure && \
       sedreplace 's/-fstack-protector/-fno-stack-protector/g' Configure
 
-   # what did that bloke think
-   grep -q -- usedl=.define hints/darwin.sh && \
-      sedreplace '/^usedl=.define.;$/d' hints/darwin.sh
-
-   preconfigure || fatal "preconfigure hook failed"
+   preconfigure
 
 #   trace configure \
    sh Configure -Duselargefiles \
@@ -296,24 +271,11 @@ EOF
       s/ *-fno-stack-protector */ /g
    ' config.sh
 
-   patchconfig || fatal "patchconfig hook failed"
-
    sh Configure -S || fatal "Configure -S failed"
 
    postconfigure || fatal "postconfigure hook failed"
 
-   : > staticstamp.configure
-}
-
-write_shellscript() {
-   {
-      echo "#!/bin/sh"
-      echo "STATICPERL=\"$STATICPERL\""
-      echo "PERL_PREFIX=\"$PERL_PREFIX\""
-      echo "MAKE=\"$MAKE\""
-      cat
-   } >"$PERL_PREFIX/bin/$1"
-   chmod 755 "$PERL_PREFIX/bin/$1"
+   touch staticstamp.configure
 }
 
 build() {
@@ -332,18 +294,6 @@ EOF
    postbuild || fatal "postbuild hook failed"
 }
 
-_postinstall() {
-   if ! [ -e "$PERL_PREFIX/staticstamp.postinstall" ]; then
-      NOCHECK_INSTALL=+
-      instcpan $STATICPERL_MODULES
-      [ $EXTRA_MODULES ] && instcpan $EXTRA_MODULES
-
-      postinstall || fatal "postinstall hook failed"
-
-      : > "$PERL_PREFIX/staticstamp.postinstall"
-   fi
-}
-
 install() {
    if ! [ -e "$PERL_PREFIX/staticstamp.install" ]; then
       build
@@ -356,8 +306,6 @@ EOF
       ln -sf "perl/bin/" "$STATICPERL/bin"
       ln -sf "perl/lib/" "$STATICPERL/lib"
 
-      mkdir "$STATICPERL/patched"
-
       ln -sf "$PERL_PREFIX" "$STATICPERL/perl" # might get overwritten
       rm -rf "$PERL_PREFIX"                    # by this rm -rf
 
@@ -366,24 +314,30 @@ EOF
       rcd "$PERL_PREFIX"
 
       # create a "make install" replacement for CPAN
-      write_shellscript SP-make-install-make <<'EOF'
-#CAT make-install-make.sh
+      cat >"$PERL_PREFIX"/bin/cpan-make-install <<EOF
+"$MAKE" || exit
+
+if find blib/arch/auto -type f | grep -q -v .exists; then
+   echo Probably an XS module, rebuilding perl
+   if "$MAKE" perl; then
+      mv perl "$PERL_PREFIX"/bin/perl~ \
+         && rm -f "$PERL_PREFIX"/bin/perl \
+         && mv "$PERL_PREFIX"/bin/perl~ "$PERL_PREFIX"/bin/perl
+      "$MAKE" -f Makefile.aperl map_clean
+   else
+      "$MAKE" -f Makefile.aperl map_clean
+      exit 1
+   fi
+fi
+
+"$MAKE" install UNINST=1
 EOF
+      chmod 755 "$PERL_PREFIX"/bin/cpan-make-install
 
-      # create a "patch modules" helper
-      write_shellscript SP-patch-postinstall <<'EOF'
-#CAT patch-postinstall.sh
-EOF
-
-      "$PERL_PREFIX/bin/SP-patch-postinstall"
-
-      # help to trick CPAN into avoiding ~/.cpan completely
+      # trick CPAN into avoiding ~/.cpan completely
       echo 1 >"$PERL_PREFIX/lib/CPAN/MyConfig.pm"
 
-      # we call cpan with -MCPAN::MyConfig in this script, which
-      # is strictly unnecssary as we have to patch CPAN anyway,
-      # so consider it "for good measure".
-      "$PERL_PREFIX"/bin/perl -MCPAN::MyConfig -MCPAN -e '
+      "$PERL_PREFIX"/bin/perl -MCPAN -e '
          CPAN::Shell->o (conf => urllist => push => "'"$CPAN"'");
          CPAN::Shell->o (conf => q<cpan_home>, "'"$STATICPERL"'/cpan");
          CPAN::Shell->o (conf => q<init>);
@@ -392,40 +346,24 @@ EOF
          CPAN::Shell->o (conf => q<prefs_dir>, "'"$STATICPERL"'/cpan/prefs");
          CPAN::Shell->o (conf => q<histfile> , "'"$STATICPERL"'/cpan/histfile");
          CPAN::Shell->o (conf => q<keep_source_where>, "'"$STATICPERL"'/cpan/sources");
-         CPAN::Shell->o (conf => q<make_install_make_command>, "'"$PERL_PREFIX"'/bin/SP-make-install-make");
+         CPAN::Shell->o (conf => q<make_install_make_command>, "'"$PERL_PREFIX"'/bin/cpan-make-install");
          CPAN::Shell->o (conf => q<prerequisites_policy>, q<follow>);
          CPAN::Shell->o (conf => q<build_requires_install_policy>, q<no>);
-         CPAN::Shell->o (conf => q<prefer_installer>, "EUMM");
          CPAN::Shell->o (conf => q<commit>);
       ' || fatal "error while initialising CPAN"
 
-      : > "$PERL_PREFIX/staticstamp.install"
+      touch "$PERL_PREFIX/staticstamp.install"
    fi
 
-   _postinstall
-}
+   if ! [ -e "$PERL_PREFIX/staticstamp.postinstall" ]; then
+      NOCHECK_INSTALL=+
+      instcpan $STATICPERL_MODULES
+      [ $EXTRA_MODULES ] && instcpan $EXTRA_MODULES
 
-import() {
-   IMPORT="$1"
+      postinstall || fatal "postinstall hook failed"
 
-   rcd "$STATICPERL"
-
-   if ! [ -e "$PERL_PREFIX/staticstamp.install" ]; then
-      verblock <<EOF
-import perl from $IMPORT to $STATICPERL
-EOF
-
-      rm -rf bin cpan lib patched perl src
-      mkdir -m 755 perl perl/bin
-      ln -s perl/bin/ bin
-      ln -s "$IMPORT" perl/bin/
-
-      echo "$IMPORT" > "$PERL_PREFIX/.import"
-
-      : > "$PERL_PREFIX/staticstamp.install"
+      touch "$PERL_PREFIX/staticstamp.postinstall"
    fi
-
-   _postinstall
 }
 
 #############################################################################
@@ -439,15 +377,11 @@ installing modules from CPAN
 $@
 EOF
 
-   MYCONFIG=
-   [ -e "$PERL_PREFIX/.import" ] || MYCONFIG=-MCPAN::MyConfig
-
-   "$PERL_PREFIX"/bin/perl $MYCONFIG -MCPAN -e 'notest (install => $_) for @ARGV' -- "$@" | tee "$STATICPERL/instcpan.log"
-
-   if grep -q " -- NOT OK\$" "$STATICPERL/instcpan.log"; then
-      fatal "failure while installing modules from CPAN ($@)"
-   fi
-   rm -f "$STATICPERL/instcpan.log"
+   for mod in "$@"; do
+      "$PERL_PREFIX"/bin/perl -MCPAN -e 'notest install => "'"$mod"'"' \
+         || fatal "$mod: unable to install from CPAN"
+   done
+   rm -rf "$STATICPERL/build"
 }
 
 #############################################################################
@@ -470,7 +404,7 @@ EOF
          "$MAKE" distclean >/dev/null 2>&1
          "$PERL_PREFIX"/bin/perl Makefile.PL || fatal "$mod: error running Makefile.PL"
          "$MAKE" || fatal "$mod: error building module"
-         "$PERL_PREFIX"/bin/SP-make-install-make install || fatal "$mod: error installing module"
+         "$PERL_PREFIX"/bin/cpan-make-install || fatal "$mod: error installing module"
          "$MAKE" distclean >/dev/null 2>&1
          exit 0
       ) || exit $?
@@ -512,7 +446,6 @@ MKBUNDLE
 }
 
 bundle() {
-   MKBUNDLE="${MKBUNDLE:=$PERL_PREFIX/bin/SP-mkbundle}"
    catmkbundle >"$MKBUNDLE~" || fatal "$MKBUNDLE~: cannot create"
    chmod 755 "$MKBUNDLE~" && mv "$MKBUNDLE~" "$MKBUNDLE"
    CACHE="$STATICPERL/cache"
@@ -530,13 +463,9 @@ if [ $# -gt 0 ]; then
          version )
             echo "staticperl version $VERSION"
             ;;
-         fetch | configure | build | install | clean | realclean | distclean )
+         fetch | configure | build | install | clean | realclean | distclean)
             ( "$command" ) || exit
             ;;
-         import )
-            ( import "$1" ) || exit
-            shift
-            ;;
          instsrc )
             ( instsrc "$@" ) || exit
             exit
@@ -545,16 +474,9 @@ if [ $# -gt 0 ]; then
             ( instcpan "$@" ) || exit
             exit
             ;;
-         perl )
-            ( install ) || exit
-            exec "$PERL_PREFIX/bin/perl" "$@"
-            exit
-            ;;
          cpan )
             ( install ) || exit
-            PERL="$PERL_PREFIX/bin/perl"
-            export PERL
-            exec "$PERL_PREFIX/bin/cpan" "$@"
+            "$PERL_PREFIX/bin/cpan" "$@"
             exit
             ;;
          mkbundle )
